"""dpypeline base package."""
__version__ = "0.1.0-beta.1"
__author__ = "Joao Morado"
__credits__ = "National Oceanography Centre"
__all__ = ["akita", "event_consumer", "etl_pipeline", "celery_app", "filesystems"]
